#!/usr/bin/env python
from gimpfu import *
import time
import datetime

# ================ *
# SERVICE ROUTINES *
# ---------------- *
#----------------------------- 
# include, then use: erro_msg 
#----------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
     
# ----------------------------- 
# to stretch all R, G, B values
# ----------------------------- 
def StretchRGBcolours (img, layer, inMin, inMax) :
    
    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    modDrawable = layer.copy()
    img.add_layer (modDrawable, pos-1)
    modDrawable.opacity = min (100.0, (inMin+inMax)*100)
    modDrawable.mode =  NORMAL_MODE
    modDrawable.name =  "StretchedRGB"

    # LEVELS STRETCH BY COLOUR

    # RED : ZERO QUOTE
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < inMin) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (modDrawable, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        toV += 1
    #gimp.message ("percent RED at value 0 = " + str(percent))
    fromMinRED = toV - 1

    #  RED : 255 QUOTE
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < inMax) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (modDrawable, HISTOGRAM_RED, fromV, toV)
        percent = percentile * 100  
        fromV -= 1
    #gimp.message ("percent RED at value 255 = " + str(percent))
    toMaxRED = fromV + 1
    
    #gimp.message ("Red From,To = "+str(fromMinRED)+","+str(toMaxRED))
    
    # GREEN : ZERO QUOTE
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < inMin) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (modDrawable, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100.0  
        toV += 1
    #gimp.message ("percent GREEN at value 0 = " + str(percent))
    fromMinGREEN = toV - 1

    # GREEN : 255 QUOTE
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < inMax) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (modDrawable, HISTOGRAM_GREEN, fromV, toV)
        percent = percentile * 100.0  
        fromV -= 1
    #gimp.message ("percent GREEN at value 255 = " + str(percent))
    toMaxGREEN = fromV + 1
   
    #gimp.message ("Green From,To = "+str(fromMinGREEN)+","+str(toMaxGREEN))
    
    # BLUE  (use reduced values) : ZERO QUOTE
    
    fromV = 0
    toV = 1
    percent = 0
    while (percent < (inMin/2)) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (modDrawable, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100.0  
        toV += 1
    #gimp.message ("percent BLUE at value 0 = " + str(percent))
    fromMinBLUE = toV - 1

    # BLUE  (use reduced values) : 255 QUOTE
    
    fromV = 254
    toV = 255
    percent = 0
    while (percent < (inMax/2)) :  
        (mean, std_dev, median, pixels, count, percentile) = pdb.gimp_histogram (modDrawable, HISTOGRAM_BLUE, fromV, toV)
        percent = percentile * 100.0  
        fromV -= 1
    #gimp.message ("percent BLUE at value 255 = " + str(percent))
    toMaxBLUE = fromV + 1
   
    #gimp.message ("Blue From,To = "+str(fromMinBLUE)+","+str(toMaxBLUE))
    
    # APPLY MODIFICATIONS TO LEVELS
    
    pdb.gimp_levels (modDrawable, 1, fromMinRED, toMaxRED, 1, 0, 255)
    pdb.gimp_levels (modDrawable, 2, fromMinGREEN, toMaxGREEN, 1, 0, 255)
    pdb.gimp_levels (modDrawable, 3, fromMinBLUE, toMaxBLUE, 1, 0, 255)
    
    return(modDrawable)

# ----------------------------- 
# to rotate R, G, B colours
# ----------------------------- 
def RotatingColours (layer, sequence) :
    
    # Indicates that the process has started.
    gimp.progress_init("Rotating colours in " + layer.name + "...")
    
    img = pdb.gimp_item_get_image (layer)

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
    newLayer = gimp.Layer(img, layer.name + " temp", layer.width, layer.height, layer.mode, layer.opacity, layer.mode)
    img.add_layer(newLayer, pos)
    layerName = layer.name
    if '.' in layerName:
        split = layerName.split(".", 1)
        layerName = split[0]
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
    # Convert the pixels to gray scale.
    try:
        # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his gray value.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        gray = (ord(pixel[0]) + ord(pixel[1]) + ord(pixel[2]))/3
                        # chosen sequence
                        # Luminance (perceived option 1): (0.299*R + 0.587*G + 0.114*B)
                        if sequence == 0:       #GBR
                            newred = blue
                            newgreen = red
                            newblue = green
                        elif sequence == 1:     #BRG
                            newred = green
                            newgreen = blue
                            newblue = red
                        elif sequence == 2:     #RBG
                            newred = red
                            newgreen = blue
                            newblue = green
                        elif sequence == 3:     #GRB
                            newred = green
                            newgreen = red
                            newblue = blue
                        else:                   #BGR
                            newred = blue
                            newgreen = green
                            newblue = red
                        #res = chr(gray) + chr(gray) + chr(gray)
                        res = chr(newred) + chr(newgreen) + chr(newblue)
                        
                        # If the image has an alpha channel (or any other channel) copy his values.
                        if(len(pixel) > 3):
                            for k in range(len(pixel)-3):
                                res += pixel[k+3]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow (True)
        newLayer.update (0, 0, newLayer.width, newLayer.height)
        suffix = ["GBR","BRG","RBG","GRB","BGR"]
        
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName+" rotated "+suffix[sequence]
    except Exception as err:
        erro_msg ("Unexpected error: " + str(err))
    
    # End progress.
    pdb.gimp_progress_end()
    
    return(newLayer)

# ================ *
#   M  A  I  N     *
# ---------------- *
   
def CrazyDrawOnDark (inImage, inBG, doMontage):
    
    pdb.gimp_context_push()
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    width = inBG.width
    height = inBG.height
    WH = width+height
    source_name = inBG.name
    if '.' in source_name:
        split = source_name.split(".", 1)
        source_name = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    RGBImage = pdb.gimp_image_new (width, height, RGB)                    
    RGBBG = pdb.gimp_layer_new_from_drawable (inBG, RGBImage)
    RGBImage.add_layer (RGBBG, 0)
    
    Base  = StretchRGBcolours (RGBImage, RGBBG, 0.25, 0.25)
    #pdb.gimp_display_new(NewImage)
    #return
    
    Base.mode = NORMAL_MODE
    Base.name = "Base: Image Layer Base for Cartoon Effect"
    Base.opacity = 100.0
    
    Cartooned = RGBBG.copy()
    RGBImage.add_layer (Cartooned, 0)
    Cartooned.mode = DIFFERENCE_MODE
    Cartooned.name = "Cartooned: Image Layer for Cartoon Effect"
    Cartooned.opacity = 100.0
    
    RADIUS=float(3.0+WH/3000.0); BLACK=float(0.100+WH/40000.000)
    pdb.plug_in_cartoon (RGBImage, Cartooned, RADIUS, BLACK)
    
    Colouring = pdb.gimp_layer_new_from_visible (RGBImage, RGBImage, "Colouring: Image Layer for more Colouring")
    RGBImage.add_layer (Colouring, 0)
    Colouring.mode = DODGE_MODE
    Colouring.opacity = 100.0
    
    Colouring2 = Colouring.copy()
    RGBImage.add_layer (Colouring2, 0)
    Colouring2.name = "Colouring2: Image Layer for more Colouring"
    Colouring3 = Colouring.copy()
    RGBImage.add_layer (Colouring3, 0)
    Colouring3.name = "Colouring3: Image Layer for more Colouring"
     
    # closing
    Outcome = pdb.gimp_image_merge_visible_layers (RGBImage, CLIP_TO_BOTTOM_LAYER)
    Outcome.name = st+" "+source_name+" Outcome of DrawOnBlack"
    
    Stretched  = StretchRGBcolours (RGBImage, Outcome, 0.25, 0.25)
    RGBBG = pdb.gimp_image_merge_down (RGBImage, Stretched, CLIP_TO_BOTTOM_LAYER)
    pdb.gimp_levels (RGBBG, HISTOGRAM_VALUE, 0, 255, 1.70, 0, 255)
    pdb.gimp_curves_spline (RGBBG, HISTOGRAM_VALUE, 8, (0,0,95,63,127,159,255,255))
    
    # ================================================================
    # outcomes in colours variations
    # ----------------------------------------------------------------
    
    variants = ["GBR","BRG","RBG","GRB","BGR"]

    GBRImage = RGBImage.duplicate()
    layer = GBRImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 0)  #"GBR"
    GBRBG = pdb.gimp_image_merge_down (GBRImage, rotated, CLIP_TO_BOTTOM_LAYER)
    GBRBG.name = name+" rotated "+variants[0]

    BRGImage = RGBImage.duplicate()
    layer = BRGImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 1)  #"BRG"
    BRGBG = pdb.gimp_image_merge_down (BRGImage, rotated, CLIP_TO_BOTTOM_LAYER)
    BRGBG.name = name+" rotated "+variants[1]

    RBGImage = RGBImage.duplicate()
    layer = RBGImage.active_layer
    name=layer.name
    rotated =  RotatingColours (layer, 2)  #"RBG"
    RBGBG = pdb.gimp_image_merge_down (RBGImage, rotated, CLIP_TO_BOTTOM_LAYER)
    RBGBG.name = name+" rotated "+variants[2]

    GRBImage = RGBImage.duplicate()
    layer = GRBImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 3)  #"GRB"
    GRBBG = pdb.gimp_image_merge_down (GRBImage, rotated, CLIP_TO_BOTTOM_LAYER)
    GRBBG.name = name+" rotated "+variants[3]

    BGRImage = RGBImage.duplicate()
    layer = BGRImage.active_layer
    name=layer.name
    rotated = RotatingColours (layer, 4)  #"BGR"
    BGRBG = pdb.gimp_image_merge_down (BGRImage, rotated, CLIP_TO_BOTTOM_LAYER)
    BGRBG.name = name+" rotated "+variants[3]
    
    if doMontage == False:
        pdb.gimp_display_new (RGBImage)
        pdb.gimp_display_new (GBRImage)
        pdb.gimp_display_new (BRGImage)
        pdb.gimp_display_new (RBGImage)
        pdb.gimp_display_new (GRBImage)
        pdb.gimp_display_new (BGRImage)
  
    # -----------------------------------------------------#
    if doMontage == True:
        around = 50
        QImage = pdb.gimp_image_new (width*3+around*4, height*2+around*3, RGB)                    
        QBG = pdb.gimp_layer_new (QImage, width*3+around*4, height*2+around*3,
                                  RGB_IMAGE, "BG for Sextet Image", 100.0, NORMAL_MODE)     
        QImage.add_layer (QBG, 0)
        gimp.set_foreground ((200,40,20))
        pdb.gimp_edit_fill (QBG, FOREGROUND_FILL)
    
        # place all siximages, the original and the 5 colours-rotated outcomes
        Q0 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 1. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q0, 0)
        pdb.gimp_edit_copy (RGBBG)
        fs = pdb.gimp_edit_paste (Q0, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "RGB"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, ADDITION_MODE)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q0 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q0, around, around)
        
        Q1 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 2. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q1, 0)
        pdb.gimp_edit_copy (GBRBG)
        fs = pdb.gimp_edit_paste (Q1, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "GBR"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, ADDITION_MODE)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q1 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q1, width+around*2, around)
         
        Q2 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 3. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q2, 0)
        pdb.gimp_edit_copy (BRGBG)
        fs = pdb.gimp_edit_paste (Q2, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "BRG"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, ADDITION_MODE)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q2 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q2, width*2+around*3, around)
        
        Q3 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 4. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q3, 0)
        pdb.gimp_edit_copy (RBGBG)
        fs = pdb.gimp_edit_paste (Q3, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "RBG"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, ADDITION_MODE)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q3 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q3, around, height+around*2)
        
        Q4 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 2. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q4, 0)
        pdb.gimp_edit_copy (GRBBG)
        fs = pdb.gimp_edit_paste (Q4, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "GRB"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, ADDITION_MODE)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q4 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q4, width+around*2, height+around*2)
        
        Q5 = pdb.gimp_layer_new (QImage, width, height, RGB_IMAGE, "To receive 3. image", 100.0, NORMAL_MODE)      
        QImage.add_layer (Q5, 0)
        pdb.gimp_edit_copy (BGRBG)
        fs = pdb.gimp_edit_paste (Q5, True)
        pdb.gimp_floating_sel_anchor (fs)
        title = "BGR"
        gimp.set_foreground ((255,255,255,255))
        Text = pdb.gimp_layer_new (QImage, width, height, RGBA_IMAGE, "Title", 100.0, ADDITION_MODE)
        QImage.add_layer (Text, 0)
        floats = pdb.gimp_text_fontname (QImage, Text, width*0.42, 2.0, title, 0, 0, width/15, 0, "Serif Bold")
        pdb.gimp_floating_sel_anchor (floats)
        Q5 = pdb.gimp_image_merge_down (QImage, Text, CLIP_TO_BOTTOM_LAYER)
        pdb.gimp_layer_translate (Q5, width*2+around*3, height+around*2)
         
         
        
        
        Sextet = pdb.gimp_image_flatten (QImage)
        Sextet.name = st+" "+source_name+" Outcome of DrawOnBlack"+" with 6 colours rotations"
        pdb.gimp_display_new (QImage)
        # ----------------------
        # delete single outcomes
        # ----------------------
        pdb.gimp_image_delete (RGBImage)
        pdb.gimp_image_delete (GBRImage)
        pdb.gimp_image_delete (BRGImage)
        pdb.gimp_image_delete (RBGImage)
        pdb.gimp_image_delete (GRBImage)
        pdb.gimp_image_delete (BGRImage)
    
    # -----------------------------------------------------------------------------------------------------
    pdb.gimp_displays_flush()
    pdb.gimp_context_pop()
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
  
    if doMontage == False:
        return(RGBBG)
    else:
        return(Sextet)
     
register(
    "CrazyDrawOnDark",
    "Gives a look of a Pencil Drawing with light contours on dark colours",
    "Gives a look of a Pencil Drawing with light contours on dark colours",
    "Diego", "Diego Nassetti ", "September 2017",
    "CrazyDrawOnDark",
    "RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_TOGGLE, "montage", "Montage Outcomes?", True),
    ],
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    CrazyDrawOnDark,
    menu="<Image>/Diego/Ancillary",
    )

main()
